#!/usr/bin/env bash

>&2 echo "Running health check and creating vitals report."

# 5 key health metrics for any website:
# 1. Security
# 2. Speed
# 3. Traffic
# 4. Usability
# 5. Conversion.  The more we leverage these throughout our conversations, the easier it is for clients to understand the context of any support task, recommendation, or new project.

## Source scripts
vendor_brewpixel_barista_bin_wp_dir=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )

source "${vendor_brewpixel_barista_bin_wp_dir}/../../Burr/burr-dependencies.sh"
ensure_readlink_installed

if [[ "$OSTYPE" == "linux-gnu" ]]; then
    vendor_brewpixel_barista_bin_dir=$(readlink -f $(dirname "${vendor_brewpixel_barista_bin_wp_dir}"))
	vendor_brewpixel_barista_dir=$(readlink -f $(dirname "${vendor_brewpixel_barista_bin_dir}"))
	vendor_brewpixel_dir=$(readlink -f $(dirname "${vendor_brewpixel_barista_dir}"))
	vendor_dir=$(readlink -f $(dirname "${vendor_brewpixel_dir}"))
	project_base=$(readlink -f $(dirname "${vendor_dir}"))
elif [[ "$OSTYPE" == "darwin"* ]]; then
	# Mac OSX
	vendor_brewpixel_barista_bin_dir=$(greadlink -f $(dirname "${vendor_brewpixel_barista_bin_wp_dir}"))
	vendor_brewpixel_barista_dir=$(greadlink -f $(dirname "${vendor_brewpixel_barista_bin_dir}"))
	vendor_brewpixel_dir=$(greadlink -f $(dirname "${vendor_brewpixel_barista_dir}"))
	vendor_dir=$(greadlink -f $(dirname "${vendor_brewpixel_dir}"))
	project_base=$(greadlink -f $(dirname "${vendor_dir}"))
else
    # Unknown.
    vendor_brewpixel_barista_bin_dir=$(readlink -f $(dirname "${vendor_brewpixel_barista_bin_wp_dir}"))
	vendor_brewpixel_barista_dir=$(readlink -f $(dirname "${vendor_brewpixel_barista_bin_dir}"))
	vendor_brewpixel_dir=$(readlink -f $(dirname "${vendor_brewpixel_barista_dir}"))
	vendor_dir=$(readlink -f $(dirname "${vendor_brewpixel_dir}"))
	project_base=$(readlink -f $(dirname "${vendor_dir}"))
fi

## Source scripts
source "${vendor_brewpixel_barista_dir}/config.sh"
source "${vendor_brewpixel_barista_dir}/Burr/bootstrap.sh"

#source "${vendor_brewpixel_barista_dir}/../bashful/bin/bashful"

for f in $(ls "${vendor_brewpixel_barista_dir}/Burr/");
    do source "${vendor_brewpixel_barista_dir}/Burr/${f}";
done

ensure_git_installed

# Bash script for checking whether WordPress is installed or not
if ! $(wp core is-installed); then
    die "WordPress is not installed yet"
fi

if [ -z MAINTENANCE_PKG_MODIFY_DATE ]; then
    MAINTENANCE_PKG_MODIFY_DATE="0 days"
fi
#today=$(date +"%Y-%m-%d")
today=$(date --date="$MAINTENANCE_PKG_MODIFY_DATE" +"%Y-%m-%d")
this_month=$(date --date="$MAINTENANCE_PKG_MODIFY_DATE" +"%B %Y")
date_range_end=$(date +"%_d %B %Y" -d ${today})

cd "${project_base}"
mkdir -p "logs/maintenance"
mkdir -p "maintenance/reports"

echo -e "![BrewPixel](../logo/logotype/png/brewpixel-logotype-black-alpha.png)\n\n" >> maintenance/reports/${today}_vitals_report.txt

## Security report
# Verify checksums

real_site_url=$(echo ${site_url} | sed -e "s/${domain_without_tld}\.local/${main_repo_name}/g")
printf "# Security Report #\n\n## _${reponame}_ ##\n\n## ${this_month} ##\n\n_${date_range_end}_\n\n**Production website:** [${reponame}](${real_site_url})  \n**Staging website:** [${reponame}.beta.tf](${real_site_url}.beta.tf)\n\n" >> printf "# Security Report #\n\n## _${reponame}_ ##\n\n## ${this_month} ##\n\n_${date_range_end}_\n\n**Production website:** [${reponame}](${real_site_url})  \n**Staging website:** [${reponame}.beta.tf](${real_site_url}.beta.tf)\n\n" >> ${today}_vitals_report.txt

verify_plugin() {
        result=$(cat "$2")
        today=$(date --date="$MAINTENANCE_PKG_MODIFY_DATE" +"%Y-%m-%d")
        verified_number=$(echo "$result" | sed 's/^.*\([0-9]\)\sof.*$/\1/')
        plugin_number=$(echo "$result" | sed 's/^.*of\s\([0-9]\).*$/\1/')
        skipped=$(echo "$result" | sed 's/^.*\(skipped\).*$/\1/')
        #echo "$verified_number" >> maintenance/reports/vitals_report.txt
        #echo "$plugin_number" >> maintenance/reports/vitals_report.txt
        if [ "$plugin_number" = "$verified_number" ]; then
            echo -e "\t✅ Success: $1 plugin verified." >> maintenance/reports/${today}_vitals_report.txt
        elif [ "$skipped" = 'skipped' ] && [ '0' = "$verified_number" ]; then
            echo -e "\tℹ️ Skipped: $1 plugin does not support verification." >> maintenance/reports/${today}_vitals_report.txt
        else
            echo -e "\t❌ Warning: $1 plugin verification failed.\n\n$result\n" >> maintenance/reports/${today}_vitals_report.txt
        fi
}
#solution to calling a function from xargs
export -f verify_plugin

if [[ "true" == ${SECURITY_PKG} ]]; then

    echo -e "\n### File Integrity Checks ###\n\nThese checks verify whether the correct files are in place and also that their content has not been changed." >> maintenance/reports/${today}_vitals_report.txt
    echo -e "\n#### File Integrity Checks Of WordPress Core ####\n\n" >> maintenance/reports/${today}_vitals_report.txt

    wp core verify-checksums | sed -e 's/^Success.*$/\t✅ &\n/' >> maintenance/reports/${today}_vitals_report.txt
    # Success: WordPress install verifies against checksums.

    echo -e "\n#### File Integrity Checks Of Wordpress Plugins ####\n\n" >> maintenance/reports/${today}_vitals_report.txt
    #@TODO: simple-page-ordering and debug-bar-list-dependencies show false positives:
    #+-----------------------------+--------------------------------------------+-----------------+
    #| plugin_name                 | file                                       | message         |
    #+-----------------------------+--------------------------------------------+-----------------+
    #| debug-bar-list-dependencies | index.php                                  | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-de_DE.mo | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-de_DE.po | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-es_ES.mo | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-es_ES.po | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-nl_NL.mo | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-nl_NL.po | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-pt_BR.mo | File is missing |
    #| simple-page-ordering        | localization/simple-page-ordering-pt_BR.po | File is missing |
    #| simple-page-ordering        | simple-page-ordering.css                   | File is missing |
    #| simple-page-ordering        | simple-page-ordering.dev.js                | File is missing |
    #| simple-page-ordering        | simple-page-ordering.js                    | File is missing |
    #| simple-page-ordering        | .DS_Store                                  | File was added  |
    #| simple-page-ordering        | assets/.DS_Store                           | File was added  |
    #+-----------------------------+--------------------------------------------+-----------------+

    #wp plugin list --field=name | grep -vE 'gravityforms|simple-page-ordering|debug-bar-list-dependencies' | xargs -I %  bash -c "echo -e '\nVerify % plugin' >> maintenance/reports/${today}_vitals_report.txt && wp plugin verify-checksums % | verify_plugin % /dev/stdin"
    wp plugin list --field=name | grep -vE 'gravityforms|simple-page-ordering|debug-bar-list-dependencies' | xargs -I %  bash -c "wp plugin verify-checksums % | verify_plugin % /dev/stdin"


    #@TODO: simple-page-ordering and debug-bar-list-dependencies show false positives so use this command when that has been solved:
    #wp plugin verify-checksums --all >> maintenance/reports/${today}_vitals_report.txt

    if wp plugin is-installed gravityforms; then
        wp plugin activate gravityformscli
        gravityformsverification=$(wp gf tool verify-checksums 2>&1)
        gravityformsverificationfail=$(echo "$gravityformsverification" | grep 'Warning\|Error')
        if [ -z "$gravityformsverification" ]; then
            echo -e "\n\t✅ Success: Gravity Forms plugin verified." >> maintenance/reports/${today}_vitals_report.txt
        elif [ -n "$gravityformsverificationfail" ]; then
            echo "$gravityformsverificationfail" | grep 'Warning\|Error'| sed 's/^/\t❌ /' &>> maintenance/reports/${today}_vitals_report.txt
        else
            echo "$gravityformsverification" >> maintenance/reports/${today}_vitals_report.txt
        fi
    fi

    inactiveplugins=$(ssh ${sshuser}@${sship} "cd "${project_base}" && wp plugin list --fields=name,status --format=csv | grep 'inactive' | sed 's/\(^.*\),.*/\t⚠️ \1 plugin is inactive./'")

    echo -e "\n### Inactive plugins ###\n" >> maintenance/reports/${today}_vitals_report.txt

    if [ -z "$inactiveplugins" ]; then
        echo -e "\t✅ Success: No inactive plugins." >> maintenance/reports/${today}_vitals_report.txt
    else
        echo -e "${inactiveplugins}" >> maintenance/reports/${today}_vitals_report.txt
        #Not using that WordPress plugin? Remove it from your installation.
        #When it comes to unused plugins, less is more. Storing unwanted plugins
        #in your WordPress installation increases the chance of a compromise,
        #even if they are disabled and not actively being used in your
        #installation.
    fi

    # Old plugins
    # Plugins not updated in 2 years should be evaluated
    
    
    #https://monitor19.sucuri.net/scan-api.php?k=82eb42946307d31e3dcfbd1fb0977f7e4783740131c6ac8907&a=scan
    # JSON output:
    #curl -q "https://monitor19.sucuri.net/api.php?k=d04aecca9f397aff4aa65f35038406c0f0e77be146c4954c52&a=query&host=http://www.sonobex.com&format=json" | jq '. | {domain: .SCAN.DOMAIN[], recommendations: .RECOMMENDATIONS, blacklist: .BLACKLIST.INFO}'
    # Human readable for report:
    #curl -q "https://monitor19.sucuri.net/api.php?k=d04aecca9f397aff4aa65f35038406c0f0e77be146c4954c52&a=query&host=http://www.cbhire.co.uk&format=json" | jq -r '. | "Domain:", .SCAN.DOMAIN[], "\nRecommendations:", .RECOMMENDATIONS, "Blacklist checks:", ([.BLACKLIST.INFO[][0]] | .[])'
    #security_scan_json=$(curl -q "${apiurl}?k=${apikey}&a=query&host=${host}&format=json" | jq '. | {domain: .SCAN.DOMAIN[], recommendations: .RECOMMENDATIONS, blacklist: .BLACKLIST.INFO}')
    #security_scan_readable=$(curl -q "${apiurl}?k=${apikey}&a=query&host=${host}&format=json" | jq -r '. | "\nDomain:", .SCAN.DOMAIN[], "\nRecommendations:", .RECOMMENDATIONS, "\n### BLACKLIST CHECK ###\n\n", ([.BLACKLIST.INFO[][0]] | .[])')
    #echo -e "$security_scan_readable" >> maintenance/reports/${today}_vitals_report.txt
#. | "\nDomain:", .SCAN.DOMAIN[], "\nRecommendations:", .RECOMMENDATIONS, "\n### BLACKLIST CHECK ###\n", ( .BLACKLIST.INFO[][1] = "" |.BLACKLIST.INFO[][0]  |=  "- " + .| [.BLACKLIST.INFO[][]] | .[])
    security_list=$(curl -q "${scanapiurl}?k=${apikey}&a=scan&host=${host}&format=json" | jq -r '. | "\n### Sucuri scans & checks ###\n\n**Scan domain:** " + .SCAN.DOMAIN[], "\n#### Web Server Security ####\n\n##### Recommendations #####\n\n" + if .RECOMMENDATIONS == null then "\t✅ Success: No web server security recommendations." else "\t⚠️ " + .RECOMMENDATIONS end, "\n#### Blacklist Checks ####\n", ( .BLACKLIST.INFO[][0]  |=  "\t✅ " + .| [.BLACKLIST.INFO[][0]] | .[])')
    #security_scan=$(curl -q "${scanapiurl}?k=${apikey}&a=scan&host=${host}&format=json" | jq -r '. | "\nDomain:", .SCAN.DOMAIN[], "\nRecommendations:", .RECOMMENDATIONS, "\n### BLACKLIST CHECK ###\n", ( .BLACKLIST.INFO[][1] |= "" | [.BLACKLIST.INFO[][]] | .[])')
    echo -e "${security_list}" >> maintenance/reports/${today}_vitals_report.txt
    #echo "" >> maintenance/reports/${today}_vitals_report.txt
    #echo -e "$security_scan" >> maintenance/reports/${today}_vitals_report.txt
    
    echo -e "\n### Default or common administrator usernames ###\n\n" >> maintenance/reports/${today}_vitals_report.txt
    # most attempts at guessing a user name will use admin
    # Background: Prior to WordPress 3.0 the username 'admin' was forced. With
    # 3.0 which was released on June 17, 2010, the default was still 'admin' but
    # a different username could be chosen. On October 24, 2013, WordPress
    # Version 3.7 was released and 'admin' was no longer the default.
    SHELL_PIPE=0 wp user list --fields=ID,user_login,display_name,user_email,roles | grep -E 'admin\s' | sed -e "1s;$; ❌ **admin** username should be changed |;" | sed -e "1s;^;| ID | user_login       | display_name         | user_email                       | roles         | notes |\n;"  | sed -e "2s;^;|----|------------------|----------------------|----------------------------------|---------------|-------|\n;"  | awk 'BEGIN{FS="|"}{print $2"\t"$3"\t"$4"\t"$5"\t"$6"\t"$7"\t"}' | awk 'BEGIN{FS="\t"}{print $1"\t| " $2"\t| " $3"\t| " $4"\t| " $5"\t| " $6}' | column -t -s $'\t' >> maintenance/reports/${today}_vitals_report.txt
    if [ "${PIPESTATUS[1]}" -eq "1" ]; then
        echo -e "\t✅  Success: No default or common administrator usernames found." >> "maintenance/reports/${today}_vitals_report.txt"
    fi
    
    # @TODO: Add check for non-administrator accounts. There should be at least one editor account to do most content edits
    # rather than only admins.
    # the username that you publish posts under should never be the admin name
    # And that user should have limited rights to the CMS
    
    # Create a nickname that’s different from your existing username and set it
    # as your public display name. This will make it more difficult for
    # attackers to brute force your login credentials.
else
    echo -e "Add BrewPixel security package for security report.\n" >> maintenance/reports/${today}_vitals_report.txt
    cat "${vendor_brewpixel_barista_dir}/inc/content/security-upgrade.md" >> maintenance/reports/${today}_vitals_report.txt
fi

# # Verify checksums for given WordPress version
# $ wp core verify-checksums --version=4.0
# Success: WordPress install verifies against checksums.
#
# # Verify checksums for given locale
# $ wp core verify-checksums --locale=en_US
# Success: WordPress install verifies against checksums.
#
# # Verify checksums for given locale
# $ wp core verify-checksums --locale=ja
# Warning: File doesn't verify against checksum: wp-includes/version.php
# Warning: File doesn't verify against checksum: readme.html
# Warning: File doesn't verify against checksum: wp-config-sample.php
# Error: WordPress install doesn't verify against checksums.

echo -e "\n---\n" >> maintenance/reports/${today}_vitals_report.txt

real_site_url=$(echo ${site_url} | sed -e "s/${domain_without_tld}\.local/${main_repo_name}/g")
printf "\n# Website Vitals and Statistics Report #\n\n## _${reponame}_ ##\n\n## ${this_month} ##\n\n_${date_range_end}_\n\n**Production website:** [${reponame}](${real_site_url})  \n**Staging website:** [${reponame}.beta.tf](${real_site_url}.beta.tf)\n\n" >> printf "# Security Report #\n\n## _${reponame}_ ##\n\n## ${this_month} ##\n\n_${date_range_end}_\n\n**Production website:** [${reponame}](${real_site_url})  \n**Staging website:** [${reponame}.beta.tf](${real_site_url}.beta.tf)\n\n" >> ${today}_vitals_report.txt

## Display WordPress version along with other information (remove package language since it does not show the active one)
echo -e "\n### WordPress ###\n" >> maintenance/reports/${today}_vitals_report.txt
wp core version --extra | grep -v language |  awk -F: '{print "**" $1 ":**" $2 "  "}' >> maintenance/reports/${today}_vitals_report.txt

#WordPress version: 4.8.3
#Database revision: 36686
#TinyMCE version:   4.310 (4310-20160418)
#Package language:  en_US

# List themes
echo -e "\n### Themes ###\n" >> maintenance/reports/${today}_vitals_report.txt
SHELL_PIPE=0 wp theme list --fields=name,status,update,version --format=table 2>>/dev/null | terminal_table_to_markdown_table >> maintenance/reports/${today}_vitals_report.txt

# List language,english_name,status fields of available languages.
echo -e "\n### Languages ###\n" >> maintenance/reports/${today}_vitals_report.txt
#SHELL_PIPE=0 wp language core list --fields=language,english_name,status,update,updated --format=table | grep -v uninstalled 2>>/dev/null | terminal_table_to_markdown_table >> maintenance/reports/${today}_vitals_report.txt
SHELL_PIPE=0 wp language core list --fields=language,english_name,status,update,updated --format=table | grep -v uninstalled | sed -e "s/|\\t/| /" | tee abc.txt | awk -v max="$(cat ${project_base}/abc.txt | wc -l)" 'BEGIN {FS="|"; OFS="\t";}{ if(NR>=4 && NR!=max)$6=substr ($6, 0, 12);if(NR>=4 && NR!=max)print $1 "|" $2 "\t|" $3 "|" $4 "\t|" $5 "|" $6 "\t|";else print}' | terminal_table_to_markdown_table >> maintenance/reports/${today}_vitals_report.txt
echo "" >> maintenance/reports/${today}_vitals_report.txt

# List all plugins on the site.
#echo -e "\n### WORDPRESS PLUGINS\n" >> maintenance/reports/${today}_vitals_report.txt

cat "${project_base}/composer.lock" | jq -r '.packages[] | [.name, .type, .version, .time] | @tsv' | sed -e "s/\\t/@\t| /g" | sed "s/\\\"//g" | column -t -s "@" | sed -e "s/\([-a-z]*\/\)\(.*\)/\2/g" | awk -v OFS='\t| ' 'BEGIN{print"Name\t| Version\t| Date Last Updated\n---\t| ---\t| ---"}; $3 ~ /wordpress-plugin/ {$3=$4="";$7=substr ($7, 0, 11);if(""==$7){$7="Date last updated information not currently supported"};if(""==$7){$7=" |"}print $1"\t" $2" " $5"\t" $6" " $7;}' | column -t -s $'\t' | awk 'BEGIN{FS="|"}{if(NR>3 && system("echo \""$3"\" | grep -q \"Date last updated information not currently supported\"") == "0" ){gsub(/^[ ]+/," ",$1);gsub(/[ \t]+$/,"",$1);$1=$1;dateupdated="curl -q https://api.wordpress.org/plugins/info/1.0/"$1".json | jq -r \" . | \".last_updated\"\""; dateupdated | getline date; $3=" "substr(date,1,10);}}{print $1"\t" $2"\t" $3;}' | column -t -s $'\t' > maintenance/reports/${today}_vitals_plugin_report.txt

cat "${project_base}/maintenance/reports/${today}_vitals_plugin_report.txt" | awk '{print $1"\t"$2"\t"$3"\t"}' | awk 'BEGIN{FS="\t"}{if(NR==1){$3="Date Last Updated";$4="Rating";}else if(NR==2){$4="---";}else if(NR>2){rating="curl -q https://api.wordpress.org/plugins/info/1.0/"$1".json | jq -r \" . | .rating\""; rating | getline rate;{if("null"!=rate){$4=rate;}else $4="Premium or bespoke plugin"}}}{print $1"\t| " $2"\t| " $3"\t| " $4}'  | column -t -s $'\t' > maintenance/reports/${today}_vitals_plugin_ratings_report.txt

cat "${project_base}/maintenance/reports/${today}_vitals_plugin_ratings_report.txt" | awk 'BEGIN{FS="|"}{print $1"\t"$2"\t"$3"\t"$4"\t"}' | awk 'BEGIN{FS="\t"}{if(NR==1){$3="Date Last Updated";$5="Downloaded";}else if(NR==2){$5="---";}else if(NR>2){gsub(/^[ ]+/," ",$1);gsub(/[ \t]+$/,"",$1);$1=$1;downloaded="curl -q https://api.wordpress.org/plugins/info/1.0/"$1".json | jq -r \" . | .downloaded\""; downloaded | getline downloads;{if("null"!=downloads){$5=int(downloads);if($5 > 1000 && $5 < 1000000) {$5=substr( $5, 0, length($5)-2 )"+ thousand"}else if($5 > 1000000 && $5 < 1000000000) {$5=int(substr( $5, 0, length($5)-5 ))"+ million"} else if($5 > 1000000000 && $5 < 1000000000000) {$5=substr( $5, 0, length($5)-8 )"+ billion"} }else $5="Premium or bespoke plugin"}}}{print $1"\t| " $2"\t| " $3"\t| " $4"\t| " $5}' | column -t -s $'\t' > maintenance/reports/${today}_vitals_plugin_downloads_report.txt

#WORKS
cat "${project_base}/maintenance/reports/${today}_vitals_plugin_downloads_report.txt" | awk 'BEGIN{FS="|"}{print $1"\t"$2"\t"$3"\t"$4"\t"$5"\t"}' | awk 'BEGIN{FS="\t"}{if(NR==1){$3="Date Last Updated";$6="Active";}else if(NR==2){$6="---";}else if(NR>2){gsub(/^[ ]+/," ",$1);gsub(/[ \t]+$/,"",$1);$1=$1;activeinstalls="curl -qg \"https://api.wordpress.org/plugins/info/1.1/?action=plugin_information&request[slug]="$1"&request[fields][active_installs]=1\" | jq -r \" . | .active_installs\""; activeinstalls | getline active;{if("null"!=active){$6=int(active);if($6 > 1000 && $6 < 1000000){$6=substr( $6, 0, length($6)-2 )","substr( $6, length($6)-2 )"+";}else if($6 > 1000000 && $6 < 1000000000) {$6=int(substr( $6, 0, length($6)-5 ))"+ million"} else if($6 > 1000000000 && $6 < 1000000000000) {$6=substr( $6, 0, length($6)-8 )"+ billion"} }else $6="Premium or bespoke plugin"}}}{print $1"\t| " $2"\t| " $3"\t| " $4"\t| " $5"\t| " $6}' | column -t -s $'\t' > maintenance/reports/${today}_vitals_plugin_active_report.txt

echo -e "\n### Plugins ###\n\n" >> maintenance/reports/${today}_vitals_report.txt
cat "${project_base}/maintenance/reports/${today}_vitals_plugin_active_report.txt" >> maintenance/reports/${today}_vitals_report.txt

#wp plugin list --format=json >> logs/maintenance/${today}_pluginreport.txt

#echo -e "\n## Plugins: Development Environment ##\n" >> maintenance/reports/${today}_vitals_report.txt
#SHELL_PIPE=0 wp plugin list --fields=name,version --format=table 2>>/dev/null | terminal_table_to_markdown_table >> maintenance/reports/${today}_vitals_report.txt
#echo "" >> maintenance/reports/${today}_vitals_report.txt

# Display the WordPress version
#echo -e "\n### WordPress version\n" >> maintenance/reports/${today}_vitals_report.txt
#wpversion=$(wp core version) && printf "%s%s\n" "WordPress version: " "$wpversion" >> maintenance/reports/${today}_vitals_report.txt
#4.8.3

# @TODO
# List roles.
#$ wp role list --fields=role --format=csv
# @TODO
# List users with only given fields
#$ wp user list --fields=display_name,user_email --format=json
#[{"display_name":"supervisor","user_email":"supervisor@gmail.com"}]
# @TODO
# List users ordered by the 'last_activity' meta value.
#$ wp user list --meta_key=last_activity --orderby=meta_value_num
>&2 echo ${domain_without_tld}
>&2 echo ${main_repo_name}
echo -e "\n### Users ###\n" >> maintenance/reports/${today}_vitals_report.txt
SHELL_PIPE=0 wp user list --fields=display_name,user_email,roles --format=table --meta_key=$(wp config get --format=table | grep table_prefix | awk '{print $2}')capabilities --orderby=meta_value,display_name 2>>/dev/null | sed -e "s/${domain_without_tld}\.local/${main_repo_name}/g" | terminal_table_to_markdown_table >> maintenance/reports/${today}_vitals_report.txt

# @TODO
# List user meta
#$ wp user meta list [user_id]
# @TODO
# List a user's sessions.
#$ wp user session list admin@example.com --format=csv
# @TODO
## List all terms associated with
#wp user term list [user_id]
# @TODO
# Get user and export to JSON file
#$ wp user get bob --format=json > bob.json

## Menus report
# List existing menus
echo -e "\n### Menus ###\n" >> maintenance/reports/${today}_vitals_report.txt
SHELL_PIPE=0 wp menu list --fields=name,count | terminal_table_to_markdown_table >> maintenance/reports/${today}_vitals_report.txt

# List available menu locations
#$ wp menu location list

# @TODO
# List all options
#$ wp option list
echo -e "\n### Option settings ###\n" >> maintenance/reports/${today}_vitals_report.txt
SHELL_PIPE=0 wp option list --fields=option_name,option_value --no-transients --exclude=wpseo* | grep -E 'WPLANG|blog_charset|blogname|current_theme|default_role|large_size_h|large_size_w|medium_large_size_h|medium_large_size_w|medium_size_h|medium_size_w|permalink_structure|template|thumbnail_size_h|thumbnail_size_w|brewpixel_typekit_code|\+--|option_name' | terminal_table_to_markdown_table | sed -e 's/[ \t]*|$/ |/' | sed -e '2s/.*/|--------------------------------------------------|---------------|/' >> maintenance/reports/${today}_vitals_report.txt

echo -e "\n### Content ###\n" >> maintenance/reports/${today}_vitals_report.txt

echo -e "\n### Posts (Most recent) ###\n" >> maintenance/reports/${today}_vitals_report.txt

has_posts=$(SHELL_PIPE=0 wp post list --post_type=post --posts_per_page=1 --format=count)

if [ "0" = "${has_posts}" ]; then
    echo -e ">There are no posts.\n" >> maintenance/reports/${today}_vitals_report.txt
else
    SHELL_PIPE=0 wp post list --post_type=post --orderby='post_date' --order=DESC --posts_per_page=10 --fields=post_status,post_title,post_date,post_modified,comment_count,comment_status,url | sed -e "s/|\\t/| /" | tee def.txt | awk -v max="$(cat ${project_base}/def.txt | wc -l)" 'BEGIN {FS="|"; OFS="|";}{ if(NR>=4 && NR!=max || NR==2){gsub(/^[ ]+/," ",$3);gsub(/[ \t]+$/,"",$3);$3=$3; if(length($3) > 35) {$3=substr($3, 0, 35)"... "; } $4=substr($4, 0, 12); if(NR>=4 && NR!=max) {cmd="date +\"%Y %b %_d\" -d \"" $4 "\""; cmd | getline four; $5=substr($5, 0, 12); cmd="date +\"%Y %b %_d\" -d \"" $5 "\""; cmd | getline five; print $1 "|" $2 "|" $3 "|" four " |" five "    |" $6 "|" $7 "|" $8 " |"; } if(NR==2) {$5=substr ($5, 0, 16); print $1 "|" $2 "|" $3 " |" $4 " |" $5 "|" $6 "|" $7 "|" $8 " |"; } } else {print} }'  | sed -e "s/http:\/\/${domain_without_tld}\.local//g" | terminal_table_to_markdown_table | sed -e "1s/post_status/Status/" | sed -e "1s/post_title/Title/" | sed -e "1s/post_date/Date/" | sed -e "1s/post_modified/Modified/" | sed -e "1s/comment_count | comment_status |/Comments ||/" | sed -e "1s/url/URL/" | sed -e '2i|  |          |   |  | Count | Status | | ' >> maintenance/reports/${today}_vitals_report.txt
    # List posts in JSON
    #$ wp post list --post_type=post --format=json
    #[{"ID":1,"post_title":"Hello world!","post_name":"hello-world","post_date":"2015-06-20 09:00:10","post_status":"publish"},{"ID":1178,"post_title":"Markup: HTML Tags and Formatting","post_name":"markup-html-tags-and-formatting","post_date":"2013-01-11 20:22:19","post_status":"draft"}]
    # Add test that will say "There are no recent posts within the last year" etc.
fi

# List all pages
echo -e "\n### Pages ###\n" >> maintenance/reports/${today}_vitals_report.txt

has_pages=$(SHELL_PIPE=0 wp post list --post_type=page --posts_per_page=1 --format=count)

if [ "0" = "${has_pages}" ]; then
    echo -e ">There are no pages.\n" >> maintenance/reports/${today}_vitals_report.txt
else
    SHELL_PIPE=0 wp post list --post_type=page --orderby=post_title --order=ASC --fields=post_status,post_title,post_date,post_modified,comment_count,comment_status,url | sed -e "s/|\\t/| /" | tee ghi.txt | awk -v max="$(cat ${project_base}/ghi.txt | wc -l)" 'BEGIN {FS="|"; OFS="|";}{ if(NR>=4 && NR!=max || NR==2){gsub(/^[ ]+/," ",$3); gsub(/[ \t]+$/,"",$3); $3=$3; if(length($3) > 35) {$3=substr($3, 0, 35)"... "; } $4=substr($4, 0, 12); if(NR>=4 && NR!=max) {cmd="date +\"%Y %b %_d\" -d \"" $4 "\""; cmd | getline four; $5=substr($5, 0, 12); cmd="date +\"%Y %b %_d\" -d \"" $5 "\""; cmd | getline five; print $1 "|" $2 "|" $3 "|" four " |" five "    |" $6 "|" $7 "|" $8 " |"; } if(NR==2) {$5=substr($5, 0, 16); print $1 "|" $2 "|" $3 " |" $4 " |" $5 "|" $6 "|" $7 "|" $8 " |"; } } else print }'  | sed -e "s/http:\/\/${domain_without_tld}\.local//g" | terminal_table_to_markdown_table | sed -e "1s/post_status/Status/" | sed -e "1s/post_title/Title/" | sed -e "1s/post_date/Date/" | sed -e "1s/post_modified/Modified/" | sed -e "1s/comment_count | comment_status |/Comments ||/" | sed -e "1s/url/URL/" | sed -e '2i|  |          |   |  | Count | Status | | ' >> maintenance/reports/${today}_vitals_report.txt
fi

# @TODO
# List registered post types
#$ wp post-type list --format=csv



# @TODO
## Comments report
# Count comments on whole blog.
#$ wp comment count
## # List approved comments.
#$ wp comment list --status=approve --fields=ID,comment_date,comment_author

# @TODO
## Config report
# List variables and constants defined in wp-config.php file.
#$ wp config get --format=table




# What caching method or plugin is being used?

# Performance test (regardless of hosting)
# Performance test (hosting aspects included)

# Deprecated functions

# Errors

# 404s

# Scan for bad themes, plugins, etc. and output our advice or warnings on report.

# Post thought inspiration:
# 
# [WordPress theme "unit tests" setup command with wp-cli](https://github.com/pixline/wp-cli-theme-test-command)
# 
# 

## Version Control / Files

#What if we already have a repo and we want to modify the `.gitignore` file to ignore files that are already in the repo?
#Show files in git index which are listed as should be ignored now:

#git ls-files -i --exclude-standard

#What if we want to see what files are being ignored?

#http://stackoverflow.com/questions/466764/show-ignored-files-in-git

#    git ls-files --others --ignored --exclude-standard

#Or in short form:

#    git ls-files -o -i --exclude-standard

>&2 echo "Finished health check and created vitals report."
